# coding: utf-8
import logging
import os
from django.conf import settings


ERROR_LOGGER_NAME = 'celery_balancer_error_logger'
INFO_LOGGER_NAME = 'celery_balancer_info_logger'


def init_logging(error_log_file='', info_log_file='', logs_path='',
                 level='info'):
    """
    :param error_log_file: имя фала логов ошибок.
    :param info_log_file: имя файла логов информации.
    :param logs_path: папка с логами.
    :param level: уровень логирования - 'info' или 'error'
    """
    if hasattr(logging, "celery_balancer_set_up_done"):
        return

    error_log_file = error_log_file or 'celery_balancer_error.log'
    info_log_file = info_log_file or 'celery_balancer_info.log'
    logs_path = logs_path or settings.LOG_PATH

    if not os.path.exists(logs_path):
        os.mkdir(logs_path)

    if level == 'info':
        loggers = [
            (ERROR_LOGGER_NAME, logging.ERROR, error_log_file),
            (INFO_LOGGER_NAME, logging.INFO, info_log_file),
        ]
    elif level == 'error':
        loggers = [
            (ERROR_LOGGER_NAME, logging.ERROR, error_log_file),
        ]
    else:
        raise Exception(u'Bad log level.')

    formatter = logging.Formatter("[%(asctime)s] %(message)s")
    formatter.datefmt = '%Y-%m-%d %H:%M:%S'
    for lname, level, fname in loggers:
        logger = logging.getLogger(lname)
        logger.setLevel(level)
        handler = logging.FileHandler(os.path.join(logs_path, fname),
                                      encoding='utf-8')
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    # Подключаем error лог к Sentry.
    error_logger = logging.getLogger(ERROR_LOGGER_NAME)
    __add_sentry_handler(error_logger)

    logging.celery_balancer_set_up_done = True


def info(msg):
    logger = logging.getLogger(INFO_LOGGER_NAME)
    logger.info(msg)


def error(msg):
    logger = logging.getLogger(ERROR_LOGGER_NAME)
    logger.error(msg, exc_info=True)


def __add_sentry_handler(logger):
    try:
        from raven.handlers.logging import SentryHandler
        from raven.conf import setup_logging, EXCLUDE_LOGGER_DEFAULTS
        sentry_handler = SentryHandler(settings.SENTRY_DSN)
        logger.addHandler(sentry_handler)
    except Exception:
        info(u'Can`t set up Sentry logging. Continue work without it.')