# coding: utf-8
from django.db import connection
from celery_balancer.helpers import dictfetchone
from celery_balancer.metrics.interface import MetricsApi
from celery_balancer.metrics.linux.os_api import LinuxOsApi


class LinuxMetricsApi(MetricsApi):
    """Класс для получения метрик в ОС Linux."""

    metrics = {
        u'mem': u'Используемая оперативная память.',
        u'swp': u'Используемый своп.',
        u'load_avg_1': u'Load average за 1 минуту.',
        u'load_avg_5': u'Load average за 5 минут.',
        u'load_avg_15': u'Load average за 15 минут.',
        u'waiting_db_procs_count': u'Количество процессов БД в '
                                   u'состоянии "waiting=True".',
        u'db_connections_count': u'Количество соединений с БД.',
    }

    def __init__(self):
        self.os_api = LinuxOsApi()
        self.db_cursor_obj = None

    def _get_mem(self, server_name):
        return self.os_api.get_mem(server_name)

    def _get_swp(self, server_name):
        return self.os_api.get_swp(server_name)

    def _get_load_avg_1(self, server_name):
        return self.os_api.get_load_avg_1(server_name)

    def _get_load_avg_5(self, server_name):
        return self.os_api.get_load_avg_5(server_name)

    def _get_load_avg_15(self, server_name):
        return self.os_api.get_load_avg_15(server_name)

    def _get_waiting_db_procs_count(self, server_name):
        self.__db_cursor.execute(
            u'SELECT COUNT(*) AS waiting_db_procs_count '
            u'FROM pg_stat_activity WHERE waiting=True'
        )
        res = dictfetchone(self.__db_cursor)
        return res['waiting_db_procs_count']

    def _get_db_connections_count(self, server_name):
        self.__db_cursor.execute(
            u'SELECT SUM(numbackends) AS db_connections_count '
            u'FROM pg_stat_database'
        )
        res = dictfetchone(self.__db_cursor)
        return res['db_connections_count']

    @property
    def __db_cursor(self):
        if self.db_cursor_obj is None:
            self.db_cursor_obj = connection.cursor()
        return self.db_cursor_obj
